/*
  Program:      Mine Mayhem
  File:         board.h
  Date: 	July, 1996 to February,   1997, version 1.00.
		26 October to 3 December,	version 2.00.
		8 December to 4 January,  2001, version 2.10.
		15 January,			version 2.11.

  Definitions for the board window.

  Freeware, Copyright 2001 Jason Hood.

  You are free to use this code, or a portion thereof, as long as an
  appropriate acknowledgement is made.

  v2.00: timing now uses uclock(), with option to use GetTicks();
	 removed all minefield code;
	 removed all inline code.
  v2.10: added cmDump() to save the board as a bitmap;
	 replaced DisplayMines() & DisplaySpaces() with DisplayProgress();
	 added tracking and auto-clear options.
*/

#ifndef _board_h
#define _board_h

#include <tws/button.h>
#include "layout.h"
#include "digits.h"
#include <time.h>


// This is queued when the right mouse button is pressed on the "face" button
#define E_BUTTONR(ID, function) \
  if (ev->Type == W_NOTIFY && ev->p1 == ID && ev->p2 == 1003) \
  { \
    function(); \
    return true; \
  }


// The "face" button in the middle of the board.
class FaceButton : public IconButton
{
  friend class BoardWindow;

public:
  FaceButton(Window* parent, ControlID ID, int x, int y, int icon);

  void ChangeIcon(int NewIcon);
  void Display(int Icon);

protected:
  void PaintWindow(int x1, int y1, int x2, int y2);
  void RButtonDown(int, int, int);

  DECLARE_RESPONSE_TABLE;

  int curicon;
};


class BoardWindow : public Window
{
public:
  BoardWindow();
  ~BoardWindow();

  void NewGame(cCoord& dim, int mines, int spaces);
  void StartTiming();
  int  GameOver(bool dead);
  void DisplayProgress(int mines, int spaces, int update);
  void DisplayTile(int x, int y, int w, int h,
		   ViewBuffer* tile, bool refresh);

protected:
  void DisplayTime(bool full = false);	// Display milliseconds if full is true
  void Display(int num, int x, int y,   // Display num using LED digits
	       ViewBuffer* bmp[] = Digit, bool zeros = false);

  void Resize(int width, int height);
  void PaintWindow(int x1, int y1, int x2, int y2);

  void cmNewGame();			// The smiley has been pressed

  void LButtonDown(int x, int y, int);	// Open or clear-around
  void RButtonDown(int x, int y, int);	// Mark or clear-around
  void MButtonDown(int x, int y, int);	// Clear-around

  void LButtonUp(int, int, int);	// Remove the highlight produced by
  void RButtonUp(int, int, int);	//  the clear-around
  void MButtonUp(int, int, int);

  void MouseMove(int x, int y, int, int but_stat);

  void Timer();                         // Update the time taken
  void cmPause(int = 0);		// Right click the smiley or use
					// the P key (hence the parameter)
  void cmDump(int = 0); 		// Save the board as a bitmap

  DECLARE_RESPONSE_TABLE;

  FaceButton* Face;			// Why didn't I call it smiley?

  int ox, oy, timex, spacesx;		// Offsets for display

  int secs, mils;			// Time taken
  uclock_t startmils, pausemils;	// Time calibration
  int oldsecs;
  bool progress;			// No progress display once game is won

  Coord olddim; 			// Previous game's dimensions

  bool isPaused;			// The game is paused
  ViewBuffer* gridsave;                 // Copy of the grid for pause
};

extern BoardWindow* Board;

#define UPDATE_MINES  1			// Values for DisplayProgress()
#define UPDATE_SPACES 2
#define UPDATE_BOTH   3

#endif
